    include 'mkl_vsl.f90'
    
    !**************************************************************************
    ! This code is written for "Smoking Initiation -- Peers and Personality",
    ! Chih-Sheng Hsieh & Hans van Kippersluis
    ! Quantitative Economics
    !
    ! Replication of Table A3 Peer Effects on Smoking -- SC-SAR Models with both 
    ! Endogenous Networks and Personality (Specification based on equation (6))
    !
    ! * users should change working directory in path1, path2, and path3
    ! * GG denotes the number of school networks
    ! * TT denotes the length of MCMC iteractions
    ! * MAX denotes the maximum size of school networks
    ! * DD denotes the dimension of latent variables
    ! * THIN denotes the thinning of the MCMC chain
    ! * Nthreads denotes the CPU threads for computing
    ! * H denotes the block size of updating latent variables
    ! * dep=2: smoking dummy
    ! * dep=3: smoking frequency
    ! * raw_norm=0: raw unnormalized weights matrix
    ! * raw_norm=1: raw normalized weights matrix
    ! * burn denotes the length of burn-in iterations
    ! * users should change initial gamma values w.r.t. DD in line 179-184
    !
    ! Please address any questions to:    
    ! Chih-Sheng Hsieh
    ! cshshieh@cuhk.edu.hk
    !***************************************************************************

    PROGRAM estimation

    USE random
    USE TOOL
    USE lapack95
    USE MKL_VSL_TYPE
    USE MKL_VSL
    USE omp_lib
    IMPLICIT NONE

    !****************** GENERAL VARIABLES ****************!
    CHARACTER(100) :: path1='/home/cshsieh/research/2014_health_study/'
    CHARACTER(100) :: path2='/data/inhome/smoke/neuroticism/'
    CHARACTER(100) :: path3='/result/SCSAR/smoke_frequency/hetero_endog_personality_without_peereffect/homo/dim4/'
    INTEGER, PARAMETER :: GG=118, TT=250000, MAX=200, DD=4, dep=3, THIN=1000, Nthreads=160, H=4, raw_norm=0, burn=100000
    REAL, PARAMETER    :: pi=3.14159265
    REAL               :: rand, kk, AICM
    INTEGER            :: NN, N(GG),  k, censor(GG), flag(GG)
    INTEGER            :: brng, method, me, i, j, g, t, d, ier,  r, seedsize, seed(2), errorflag, INFO, seed_mkl, t_round
    INTEGER            :: clock_max, clock_rate, clock_start, clock_stop, ithread, iblock,ibegin,iend
    REAL, DIMENSION(MAX,MAX,GG):: grade, sex, race, network, W_norm, W_FF, W_FM, W_MF, W_MM
    REAL, DIMENSION(MAX,18,GG) :: group
    REAL, ALLOCATABLE:: C1(:,:), C2(:,:), C3(:,:), W(:,:)
    REAL, ALLOCATABLE:: Y(:), X1(:,:), X2(:,:)
    REAL, ALLOCATABLE:: Nu(:), Co(:), Ex(:)
    INTEGER, ALLOCATABLE :: INDX(:), p_dim(:), IPVT(:), IPIV(:)
    LOGICAL            :: first=.true.
    TYPE (VSL_STREAM_STATE) :: stream
    INTEGER ::  errcode
    INTEGER :: CODE, SEED_PERM
    REAL(8)    :: det1, det2, pp


    !*** hyperparameters in the prior distribution ***!
    REAL, DIMENSION(7+DD)           :: gamma_0
    REAL, DIMENSION(30+2*DD)        :: beta_10
    REAL, DIMENSION(10+DD)          :: beta_20
    REAL, DIMENSION(7+DD,7+DD)      :: G_0
    REAL, DIMENSION(30+2*DD,30+2*DD):: B_10, INV_B_10
    REAL, DIMENSION(10+DD,10+DD)    :: B_20, INV_B_20
    REAL                            :: sigma_0
    REAL                            :: Sig_0
    REAL                            :: ALPHA_0, a_0

    !*** posterior draws of parameters *******!

    REAL, DIMENSION(7+DD,TT)       :: gamma_T
    REAL, DIMENSION(TT)            :: lambda_T
    REAL, DIMENSION(30+2*DD, TT)   :: beta_yT
    REAL, DIMENSION(10+DD, TT)     :: beta_nT, beta_cT, beta_eT
    REAL, DIMENSION(TT)            :: sigma_eT, sigma_v1T, sigma_v2T, sigma_v3T
    REAL(8), DIMENSION(TT)    :: logp_T
    REAL, DIMENSION(GG,TT)    :: alpha_yT,alpha_nT,alpha_cT,alpha_eT
    REAL                      :: c_0(GG), c_1, c_2, c_3,c_4,c_5,c_6,c_7,c_8,c_9, acc_0(GG), acc_0v
    REAL                      :: acc_1, acc_2, acc_3, acc_4, acc_5, acc_6, acc_7, acc_8, acc_9
    REAL, DIMENSION(GG,TT)    :: acc_rate0
    REAL, DIMENSION(TT)       :: acc_rate1, acc_rate2, acc_rate3, acc_rate4, acc_rate5, acc_rate6
    REAL, DIMENSION(TT)       :: acc_rate7, acc_rate8, acc_rate9

    !******  VARIABLE IN M-H FOR Z_G  *******************************!

    REAL, DIMENSION(MAX,DD,GG):: zz
    REAL, ALLOCATABLE  :: zz_1(:,:), zz_2(:,:), cov_zz(:,:), temp_zz(:)
    REAL, ALLOCATABLE  :: S(:,:), S1(:,:), S2(:,:)
    REAL, ALLOCATABLE  :: Vl(:,:), Vn(:,:), Vc(:,:), Ve(:,:), V1(:,:), V2(:,:)
    REAL, ALLOCATABLE  :: psi(:,:), psi_1(:,:), psi_2(:,:), zero(:)
    REAL, ALLOCATABLE  :: ep(:), ep_1(:), ep_2(:), ep_1v(:), ep_2v(:)
    REAL, ALLOCATABLE  :: ep_n(:), ep_n1(:), ep_n2(:), ep_n1v(:), ep_n2v(:)
    REAL, ALLOCATABLE  :: ep_c(:), ep_c1(:), ep_c2(:), ep_c1v(:), ep_c2v(:)
    REAL, ALLOCATABLE  :: ep_e(:), ep_e1(:), ep_e2(:), ep_e1v(:), ep_e2v(:)
    REAL(8) :: q_1, q_2, pp_w, q
    REAL(8) :: like_Y1, like_Y2, like_N1, like_N2, like_C1, like_C2, like_E1, like_E2

    !REAL, DIMENSION(MAX,DD,GG,TT/THIN):: zz_T

    !********VARIABLE IN M-H FOR GAMMA & LAMBDA   ***************!

    REAL      :: lambda_1, lambda_2, COV_lambda_1, lambda_0
    REAL      :: gamma_1(7+DD), gamma_2(7+DD), COV_gamma_1(7+DD,7+DD)
    REAL(8)   :: pp_l(GG), pp_n(GG), pp_c(GG), pp_e(GG), pp_G(GG), pp_sig(GG)

    !*************VARIABLE IN M-H FOR BETA **************!

    REAL, ALLOCATABLE :: XVX(:,:,:), XVY(:,:), INV_B1(:,:), INV_B2(:,:), beta_temp(:)
    REAL, ALLOCATABLE ::  XX1(:,:), XX2(:,:), YY(:), FACT(:,:)

    !*************VARIABLE IN M-H FOR SIGMA_E & SIGMA_EZ & ALPHA_G**************!

    REAL              :: sigma_1(1+DD), sigma_2(1+DD), COV_sigma_1(1+DD,1+DD)
    REAL              :: R_g,  eta_0, ep_v(GG)
    REAL, ALLOCATABLE :: VV1(:,:), VV2(:,:)
    INTEGER           :: rho_0, rho_1, stack

    !*******************************************************************************************************!

    INTEGER :: N_1, N_2, T_1, T_2
    REAL, ALLOCATABLE:: rowsum_w_ff(:), rowsum_w_fm(:), rowsum_w_mf(:), rowsum_w_mm(:), rowsum_w_norm(:)

    !*******************************************************************************************************!

100 format (14f10.4)
200 format (4f10.4)

    seedsize=2
    seed(1)=2.1103e+005
    seed(2)=2.1103e+005
    CALL RANDOM_SEED
    CALL RANDOM_SEED(SIZE=seedsize)
    CALL RANDOM_SEED(PUT=seed)

    seed_mkl=12345
    brng=VSL_BRNG_MCG31
    method=VSL_RNG_METHOD_GAUSSIANMV_BOXMULLER
    me=VSL_MATRIX_STORAGE_FULL

    errcode=vslnewstream(stream,brng,seed_mkl)

    CALL networksize(GG, MAX, path1, path2, N)
    CALL define_variable(GG, grade, sex, race, network, group, N, path1, path2, MAX)

    W_norm=0.0
    W_FF=0.0
    W_FM=0.0
    W_MF=0.0
    W_MM=0.0
    zz=0.0
    lambda_0=0.0
    gamma_0=0.0
    beta_10=0.0
    beta_20=0.0
    rho_0=2
    eta_0=0.1
    G_0=eye(7+DD)*100.0
    Sig_0=100.0
    B_10=eye(30+2*DD)*100.0
    B_20=eye(10+DD)*100.0
    CALL FINDInv(B_10,INV_B_10,30+2*DD,ier)
    CALL FINDInv(B_20,INV_B_20,10+DD,ier)
    zz=0.0
    ALPHA_0=400.0
    a_0=0.0
    censor=0

    c_0=0.1
    acc_1=0.0;acc_2=0.0;acc_3=0.0;acc_4=0.0;acc_5=0.0;acc_6=0.0;
    acc_rate1=0.0;acc_rate2=0.0;acc_rate3=0.0;acc_rate4=0.0;acc_rate5=0.0;acc_rate6=0.0;

    ! initial value of the draws !
    !gamma_T(:,1)=(/ -3.2356  ,  2.3110   , 0.3811  ,  0.8758 ,  -0.0375 ,  -0.0257 ,  -0.2598  , 10.1573/)   ! D=1 !
    !gamma_T(:,1)=(/   -1.5175  ,  2.4617 ,   0.3530  ,  0.9279 ,  -0.0608 ,  -0.0296  , -0.2535 ,   6.0569  ,  5.0701 /)  ! D=2 !
    !gamma_T(:,1)=(/    -0.7762  ,  2.5907  ,  0.3542  ,  1.0207  ,  0.2556 ,   0.0985 ,  -0.1097  ,  5.1491 ,   4.7550   , 2.0477/) !D=3!
    gamma_T(:,1)=(/   0.4440  ,  2.7533  ,  0.3498  ,  1.1216  , 0.2000  , 0.1400,   -0.1654  ,  4.5842  ,  2.6175  ,  2.5778  ,  2.5179/) !D=4!
    !gamma_T(:,1)=(/  1.2131  ,  2.7815  ,  0.3553  ,  1.1301 ,   0.2144  ,  0.0744 ,  -0.1529  ,  3.2928  ,  3.1720  ,  3.0632  ,  2.5408 ,   0.8042  /) !D=5!
    !gamma_T(:,1)=(/    1.9974 ,   2.8471 ,   0.3454  ,  1.1863  ,  0.8274  ,  0.0259  , -0.1388  ,  3.7865  ,  3.6299  ,  3.3746  ,  3.0000  ,  1.3158 ,  1.3000/) !D=6!
    sigma_eT(1)=0.2
    sigma_v1T(1)=0.4
    sigma_v2T(1)=0.6
    sigma_v3T(1)=0.6

    OPEN (1000, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'gamma.txt', status='unknown')
    OPEN (2000, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'lambda.txt', status='unknown')
    OPEN (3000, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'beta.txt', status='unknown')
    OPEN (3100, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'beta_n.txt', status='unknown')
    OPEN (3200, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'beta_c.txt', status='unknown')
    OPEN (3300, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'beta_e.txt', status='unknown')
    OPEN (4000, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'sigma.txt', status='unknown')
    OPEN (4100, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'sigma_n.txt', status='unknown')
    OPEN (4200, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'sigma_c.txt', status='unknown')
    OPEN (4300, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'sigma_e.txt', status='unknown')
    !OPEN (5000, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'latent_z.txt', status='replace')
    !OPEN (6000, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'alpha.txt', status='unknown')
    OPEN (7000, file =  trim(adjustl(path1)) // trim(adjustl(path3)) // 'loglikeli.txt', status='unknown')


    ! read raw data and sort by personality -> four spatial weight matrix !

    DO g=1,GG
        N_1=0
        N_2=0
        NN=N(g)
        ALLOCATE(W(NN,NN));
        ALLOCATE(p_dim(NN));
        ALLOCATE(rowsum_w_ff(NN))
        ALLOCATE(rowsum_w_fm(NN))
        ALLOCATE(rowsum_w_mf(NN))
        ALLOCATE(rowsum_w_mm(NN))
        ALLOCATE(rowsum_w_norm(NN))

        p_dim=group(1:NN,1,g)
        W=network(1:NN,1:NN,g)

        DO i=1,NN
            IF (p_dim(i)==0) N_1=N_1+1
            IF (p_dim(i)==1) N_2=N_2+1
        END DO
        T_1=N_1
        T_2=N_1+N_2

        w_ff(1:T_1,1:T_1,g)=W(1:T_1,1:T_1)
        w_fm(1:T_1,T_1+1:T_2,g)=W(1:T_1,T_1+1:T_2)
        w_mf(T_1+1:T_2,1:T_1,g)=W(T_1+1:T_2,1:T_1)
        w_mm(T_1+1:T_2,T_1+1:T_2,g)=W(T_1+1:T_2,T_1+1:T_2)
        w_norm(1:NN,1:NN,g)=W

        if (raw_norm==1) THEN
            rowsum_w_ff=sum(w_ff(1:NN,1:NN,g), dim=2)
            rowsum_w_fm=sum(w_fm(1:NN,1:NN,g), dim=2)
            rowsum_w_mf=sum(w_mf(1:NN,1:NN,g), dim=2)
            rowsum_w_mm=sum(w_mm(1:NN,1:NN,g), dim=2)
            rowsum_w_norm=sum(W, dim=2)
            DO i=1,NN
                if (rowsum_w_ff(i)==0) rowsum_w_ff(i)=1;
                if (rowsum_w_fm(i)==0) rowsum_w_fm(i)=1;
                if (rowsum_w_mf(i)==0) rowsum_w_mf(i)=1;
                if (rowsum_w_mm(i)==0) rowsum_w_mm(i)=1;
                if (rowsum_w_norm(i)==0) rowsum_w_norm(i)=1;

                w_ff(i,1:NN,g)=w_ff(i,1:NN,g)/rowsum_w_ff(i);
                w_fm(i,1:NN,g)=w_fm(i,1:NN,g)/rowsum_w_fm(i);
                w_mf(i,1:NN,g)=w_mf(i,1:NN,g)/rowsum_w_mf(i);
                w_mm(i,1:NN,g)=w_mm(i,1:NN,g)/rowsum_w_mm(i);
                w_norm(i,1:NN,g)=W(i,1:NN)/rowsum_w_norm(i);
            END DO
        END IF
        DEALLOCATE(W);
        DEALLOCATE(p_dim);
        DEALLOCATE(rowsum_w_ff)
        DEALLOCATE(rowsum_w_fm)
        DEALLOCATE(rowsum_w_mf)
        DEALLOCATE(rowsum_w_mm)
        DEALLOCATE(rowsum_w_norm)

    END DO

    ! Start MCMC sampling !

    DO t=2, TT

        call system_clock ( clock_start, clock_rate, clock_max )

        DO
            lambda_1=random_normal()*0.1+lambda_T(t-1)
            IF (t>2) THEN
                lambda_2=random_normal()*sqrt(var(lambda_T(1:t-1),t-1))*2.38+lambda_T(t-1)
                lambda_1=0.95*lambda_2+0.05*lambda_1
            END IF
            IF ((lambda_1>=-0.10) .and. (lambda_1<=0.10)) EXIT
        END DO

        DO
            COV_gamma_1=EYE(7+DD)*0.0001/(7+DD)
            call spotrf('U',7+DD, COV_gamma_1, 7+DD, info)
            errcode=vsrnggaussianmv(method,stream,1,gamma_1, 7+DD, me, gamma_0, COV_gamma_1)
            gamma_1=gamma_1+gamma_T(:,t-1)
            IF (t>200) THEN
                COV_gamma_1=cov(gamma_T(:,1:t-1),7+DD,t-1)*2.38**2/(7+DD)
                call spotrf('U',7+DD, COV_gamma_1, 7+DD, info)
                errcode=vsrnggaussianmv(method,stream,1,gamma_2, 7+DD, me, gamma_0, COV_gamma_1)
                gamma_2=gamma_2+gamma_T(:,t-1)
                gamma_1=0.5*gamma_2+0.5*gamma_1
            END IF
            IF (DD==1) THEN
                IF (gamma_1(7+DD)>=0) EXIT
            ELSEIF (DD==2) THEN
                IF (gamma_1(6+DD)>=gamma_1(7+DD)) EXIT
            ELSEIF (DD==3) THEN
                IF ((gamma_1(5+DD)>=gamma_1(6+DD)) .and. (gamma_1(6+DD)>=gamma_1(7+DD)) .and. (gamma_1(7+DD)>=0)) EXIT
            ELSEIF (DD==4)  THEN
                IF ((gamma_1(4+DD)>=gamma_1(5+DD)) .and. (gamma_1(5+DD)>=gamma_1(6+DD)) &
                    &  .and. (gamma_1(6+DD)>=gamma_1(7+DD)) .and. (gamma_1(7+DD)>=0)  ) EXIT
            ELSEIF (DD==5)  THEN
                IF ((gamma_1(3+DD)>=gamma_1(4+DD)) .and. (gamma_1(4+DD)>=gamma_1(5+DD)) &
                    & .and. (gamma_1(5+DD)>=gamma_1(6+DD)) .and. (gamma_1(6+DD)>=gamma_1(7+DD)) .and. (gamma_1(7+DD)>=0) ) EXIT
            ELSEIF (DD==6)  THEN
                IF ((gamma_1(2+DD)>=gamma_1(3+DD)) .and. (gamma_1(3+DD)>=gamma_1(4+DD))  &
                    &  .and. (gamma_1(4+DD)>=gamma_1(5+DD)) .and. (gamma_1(5+DD)>=gamma_1(6+DD)) .and. (gamma_1(6+DD)>=gamma_1(7+DD)) .and. (gamma_1(7+DD)>=0.0) ) EXIT
            END IF
        END DO

        gamma_2=gamma_T(:,t-1)

        ! THE M-H ALGORITHM FOR SAMPLING Z !


        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g, NN, Y, Nu, Co, Ex, X1, XX1, X2, XX2, W, &
        !$omp & S, S1, S2, &
        !$omp & psi_1, psi_2, ep, ep_1, ep_2, ep_1v, ep_2v, INFO, CODE, IPVT, FACT, &
        !$omp & ep_n, ep_n1, ep_n2, ep_n1v, ep_n2v, ep_c, ep_c1, ep_c2, ep_c1v, ep_c2v, ep_e, ep_e1, ep_e2, ep_e1v, ep_e2v, &
        !$omp & C1, C2, C3, Vl, Vn, Vc, Ve, V1, V2, zz_1, zz_2, IPIV, INDX, DET1, DET2, q_1, q_2, &
        !$omp & like_Y1, like_Y2, like_N1, like_N2, like_C1, like_C2, like_E1, like_E2,  &
        !$omp & kk, acc_0v, k, cov_zz, temp_zz, zero, d, pp, i, j, rand, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Y(NN));           ALLOCATE(Nu(NN));
            ALLOCATE(Co(NN));          ALLOCATE(Ex(NN));
            ALLOCATE(W(NN,NN));
            ALLOCATE(Vl(NN,NN));       ALLOCATE(Vn(NN,NN));
            ALLOCATE(Vc(NN,NN));       ALLOCATE(Ve(NN,NN));
            ALLOCATE(S(NN,NN));
            ALLOCATE(S1(NN,NN));       ALLOCATE(S2(NN,NN));
            ALLOCATE(X1(NN,15)) ;      ALLOCATE(XX1(NN,30+2*DD)) ;
            ALLOCATE(X2(NN,10)) ;      ALLOCATE(XX2(NN,10+DD)) ;
            ALLOCATE(psi_1(NN,NN));    ALLOCATE(psi_2(NN,NN))
            ALLOCATE(ep(NN));
            ALLOCATE(ep_1(NN));        ALLOCATE(ep_2(NN));
            ALLOCATE(ep_1v(NN));       ALLOCATE(ep_2v(NN));
            ALLOCATE(ep_n(NN));
            ALLOCATE(ep_n1(NN));        ALLOCATE(ep_n2(NN));
            ALLOCATE(ep_n1v(NN));       ALLOCATE(ep_n2v(NN));
            ALLOCATE(ep_c(NN));
            ALLOCATE(ep_c1(NN));        ALLOCATE(ep_c2(NN));
            ALLOCATE(ep_c1v(NN));       ALLOCATE(ep_c2v(NN));
            ALLOCATE(ep_e(NN));
            ALLOCATE(ep_e1(NN));        ALLOCATE(ep_e2(NN));
            ALLOCATE(ep_e1v(NN));       ALLOCATE(ep_e2v(NN));
            ALLOCATE(C1(NN,NN));       ALLOCATE(C2(NN,NN));
            ALLOCATE(C3(NN,NN));
            ALLOCATE(V1(NN,NN));       ALLOCATE(V2(NN,NN))  ;
            ALLOCATE(zz_1(NN,DD));     ALLOCATE(zz_2(NN,DD))
            ALLOCATE(IPIV(NN));        ALLOCATE(INDX(NN));
            ALLOCATE(IPVT(NN));        ALLOCATE(FACT(NN,NN));


            Y=group(1:NN,dep,g)
            Nu=group(1:NN,4,g)
            Co=group(1:NN,5,g)
            Ex=group(1:NN,6,g)
            X1=group(1:NN,4:18,g)
            X2=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            XX1(:,1:15)=X1
            XX1(:,16:30)=matmul(w_norm(1:NN,1:NN,g),X1)
            XX2(:,1:10)=X2
            zz_1=zz(1:NN,1:DD,g)
            zz_2=zz(1:NN,1:DD,g)

            C1=grade(1:NN,1:NN,g)
            C2=sex(1:NN,1:NN,g)
            C3=race(1:NN,1:NN,g)

            S=eye(NN)-lambda_T(t-1)*w_norm(1:NN,1:NN,g)

            kk=NN
            kk=floor(kk/H)
            IF (kk==0) kk=kk+1

            acc_0v=0.0
            DO k=1, kk
                IF(k==kk) THEN
                    ALLOCATE(cov_zz(NN-(k-1)*H,NN-(k-1)*H))
                    ALLOCATE(temp_zz(NN-(k-1)*H))
                    ALLOCATE(zero(NN-(k-1)*H))
                    zero=0.0
                    cov_zz=EYE(NN-(k-1)*H)*c_0(g)
                    call spotrf('U',NN-(k-1)*H, cov_zz, NN-(k-1)*H, info)
                    DO d=1, DD
                        errcode=vsrnggaussianmv(method,stream, 1, temp_zz, NN-(k-1)*H, me, zero, cov_zz)
                        zz_1((k-1)*H+1:NN,d)=temp_zz+zz_2((k-1)*H+1:NN,d)
                    END DO
                    DEALLOCATE(cov_zz)
                    DEALLOCATE(temp_zz)
                    DEALLOCATE(zero)
                ELSE
                    ALLOCATE(cov_zz(H,H))
                    ALLOCATE(temp_zz(H))
                    ALLOCATE(zero(H))
                    zero=0.0
                    cov_zz=EYE(H)*c_0(g)
                    call spotrf('U',H, cov_zz, H, info)
                    DO d=1,DD
                        errcode=vsrnggaussianmv(method,stream, 1, temp_zz, H, me, zero, cov_zz)
                        zz_1((k-1)*H+1:k*H,d)=temp_zz+zz_2((k-1)*H+1:k*H,d)
                    END DO
                    DEALLOCATE(cov_zz)
                    DEALLOCATE(temp_zz)
                    DEALLOCATE(zero)
                END IF
                pp=0.0
                DO i=1,NN
                    DO j=1,NN
                        IF (j/=i) THEN
                            IF (DD==1) THEN
                                psi_1(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j) &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_1(i,1)-zz_1(j,1))
                                psi_2(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j) &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_2(i,1) -zz_2(j,1))
                            ELSEIF (DD==2) THEN
                                psi_1(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j) &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t-1)*abs(zz_1(i,2)-zz_1(j,2))
                                psi_2(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j) &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_2(i,1)-zz_2(j,1))-gamma_T(9,t-1)*abs(zz_2(i,2)-zz_2(j,2))
                            ELSEIF (DD==3) THEN
                                psi_1(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_1(i,1)-zz_1(j,1)) -gamma_T(9,t-1)*abs(zz_1(i,2)-zz_1(j,2)) &
                                    & -gamma_T(10,t-1)*abs(zz_1(i,3)-zz_1(j,3))
                                psi_2(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_2(i,1)-zz_2(j,1)) -gamma_T(9,t-1)*abs(zz_2(i,2)-zz_2(j,2)) &
                                    & -gamma_T(10,t-1)*abs(zz_2(i,3)-zz_2(j,3))
                            ELSEIF (DD==4) THEN
                                psi_1(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t-1)*abs(zz_1(i,2)-zz_1(j,2)) &
                                    & -gamma_T(10,t-1)*abs(zz_1(i,3)-zz_1(j,3))-gamma_T(11,t-1)*abs(zz_1(i,4)-zz_1(j,4))
                                psi_2(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_2(i,1)-zz_2(j,1)) -gamma_T(9,t-1)*abs(zz_2(i,2)-zz_2(j,2))  &
                                    & -gamma_T(10,t-1)*abs(zz_2(i,3)-zz_2(j,3)) -gamma_T(11,t-1)*abs(zz_2(i,4)-zz_2(j,4))
                            ELSEIF (DD==5) THEN
                                psi_1(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t-1)*abs(zz_1(i,2)-zz_1(j,2)) &
                                    & -gamma_T(10,t-1)*abs(zz_1(i,3)-zz_1(j,3))-gamma_T(11,t-1)*abs(zz_1(i,4)-zz_1(j,4)) &
                                    & -gamma_T(12,t-1)*abs(zz_1(i,5)-zz_1(j,5))
                                psi_2(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_2(i,1)-zz_2(j,1)) -gamma_T(9,t-1)*abs(zz_2(i,2)-zz_2(j,2))  &
                                    & -gamma_T(10,t-1)*abs(zz_2(i,3)-zz_2(j,3)) -gamma_T(11,t-1)*abs(zz_2(i,4)-zz_2(j,4)) &
                                    & -gamma_T(12,t-1)*abs(zz_2(i,5)-zz_2(j,5))
                            ELSEIF (DD==6) THEN
                                psi_1(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t-1)*abs(zz_1(i,2)-zz_1(j,2)) &
                                    & -gamma_T(10,t-1)*abs(zz_1(i,3)-zz_1(j,3))-gamma_T(11,t-1)*abs(zz_1(i,4)-zz_1(j,4)) &
                                    & -gamma_T(12,t-1)*abs(zz_1(i,5)-zz_1(j,5))-gamma_T(13,t-1)*abs(zz_1(i,6)-zz_1(j,6))
                                psi_2(i,j)=gamma_T(1,t-1)+gamma_T(2,t-1)*C1(i,j)+gamma_T(3,t-1)*C2(i,j)+gamma_T(4,t-1)*C3(i,j)  &
                                    & +gamma_T(5,t-1)*abs(Nu(i)-Nu(j))+gamma_T(6,t-1)*abs(Co(i)-Co(j))+gamma_T(7,t-1)*abs(Ex(i)-Ex(j)) &
                                    & -gamma_T(8,t-1)*abs(zz_2(i,1)-zz_2(j,1)) -gamma_T(9,t-1)*abs(zz_2(i,2)-zz_2(j,2))  &
                                    & -gamma_T(10,t-1)*abs(zz_2(i,3)-zz_2(j,3)) -gamma_T(11,t-1)*abs(zz_2(i,4)-zz_2(j,4)) &
                                    & -gamma_T(12,t-1)*abs(zz_2(i,5)-zz_2(j,5))-gamma_T(13,t-1)*abs(zz_2(i,6)-zz_2(j,6))
                            ENDIF

                            q_1=psi_1(i,j)*W(i,j)-log(1+exp(psi_1(i,j)))
                            q_2=psi_2(i,j)*W(i,j)-log(1+exp(psi_2(i,j)))
                            pp=pp+(q_1-q_2)
                        ENDIF
                    END DO
                END DO

                !****************************************************

                XX1(:,31:30+DD)=zz_1
                XX1(:,31+DD:30+2*DD)=matmul(w_norm(1:NN,1:NN,g),zz_1)

                ep_1=matmul(S,Y)-matmul(XX1,beta_yT(:,t-1))-alpha_yT(g,t-1)

                XX1(:,31:30+DD)=zz_2
                XX1(:,31+DD:30+2*DD)=matmul(w_norm(1:NN,1:NN,g),zz_2)

                ep_2=matmul(S,Y)-matmul(XX1,beta_yT(:,t-1))-alpha_yT(g,t-1)


                like_Y1=-dot_product(ep_1, ep_1)/(2*sigma_eT(t-1))
                like_Y2=-dot_product(ep_2, ep_2)/(2*sigma_eT(t-1))


                !*****************************************************
                XX2(:,11:10+DD)=zz_1

                ep_n1=Nu-matmul(XX2,beta_nT(:,t-1))-alpha_nT(g,t-1)
                ep_c1=Co-matmul(XX2,beta_cT(:,t-1))-alpha_cT(g,t-1)
                ep_e1=Ex-matmul(XX2,beta_eT(:,t-1))-alpha_eT(g,t-1)

                XX2(:,11:10+DD)=zz_2

                ep_n2=Nu-matmul(XX2,beta_nT(:,t-1))-alpha_nT(g,t-1)
                ep_c2=Co-matmul(XX2,beta_cT(:,t-1))-alpha_cT(g,t-1)
                ep_e2=Ex-matmul(XX2,beta_eT(:,t-1))-alpha_eT(g,t-1)

                like_N1=-dot_product(ep_n1, ep_n1)/(2*sigma_v1T(t-1))
                like_N2=-dot_product(ep_n2, ep_n2)/(2*sigma_v1T(t-1))

                like_C1=-dot_product(ep_c1, ep_c1)/(2*sigma_v2T(t-1))
                like_C2=-dot_product(ep_c2, ep_c2)/(2*sigma_v2T(t-1))

                like_E1=-dot_product(ep_e1, ep_e1)/(2*sigma_v3T(t-1))
                like_E2=-dot_product(ep_e2, ep_e2)/(2*sigma_v3T(t-1))


                IF (k==kk) THEN
                    pp=pp+(like_Y1-like_Y2)+(like_N1-like_N2)+(like_C1-like_C2)+(like_E1-like_E2) &
                        & +logmvnpdf(reshape(zz_1((k-1)*H+1:NN,:),(/(NN-(k-1)*H)*DD,1/)),eye((NN-(k-1)*H)*DD),(NN-(k-1)*H)*DD)-logmvnpdf(reshape(zz_2((k-1)*H+1:NN,:),(/(NN-(k-1)*H)*DD,1/)),eye((NN-(k-1)*H)*DD),(NN-(k-1)*H)*DD)
                ELSE
                    pp=pp+(like_Y1-like_Y2)+(like_N1-like_N2)+(like_C1-like_C2)+(like_E1-like_E2) &
                        & +logmvnpdf(reshape(zz_1((k-1)*H+1:k*H,:),(/H*DD,1/)),eye(H*DD),H*DD)-logmvnpdf(reshape(zz_2((k-1)*H+1:k*H,:),(/H*DD,1/)),eye(H*DD),H*DD)
                END IF

                CALL random_number(rand)
                IF (log(rand)<=pp) THEN
                    zz_2=zz_1
                    acc_0v=acc_0v+1
                END IF
                zz_1=zz_2
            END DO
            if (3*acc_0v>1*kk) acc_0(g)=acc_0(g)+1
            acc_rate0(g,t)=acc_0(g)/t
            if ((acc_rate0(g,t)<0.2) )  c_0(g)=c_0(g)/1.01
            if ((acc_rate0(g,t)>0.3) )  c_0(g)=c_0(g)*1.01
            zz(1:NN,1:DD,g)=zz_1

            !***** THE M-H ALGORITHM FOR SAMPLING GAMMA & LAMBDA & RHO ******!

            pp=0.0
            DO i=1,NN
                DO j=1, NN
                    IF (j/=i) THEN
                        IF (DD==1) THEN
                            psi_1(i,j)=gamma_1(1)+gamma_1(2)*c1(i,j)+gamma_1(3)*c2(i,j)+gamma_1(4)*c3(i,j) &
                                & +gamma_1(5)*abs(Nu(i)-Nu(j))+gamma_1(6)*abs(Co(i)-Co(j))+gamma_1(7)*abs(Ex(i)-Ex(j))-gamma_1(8)*abs(zz_1(i,1)-zz_1(j,1))
                            psi_2(i,j)=gamma_2(1)+gamma_2(2)*c1(i,j)+gamma_2(3)*c2(i,j)+gamma_2(4)*c3(i,j) &
                                & +gamma_2(5)*abs(Nu(i)-Nu(j))+gamma_2(6)*abs(Co(i)-Co(j))+gamma_2(7)*abs(Ex(i)-Ex(j))-gamma_2(8)*abs(zz_1(i,1)-zz_1(j,1))
                        ELSEIF (DD==2) THEN
                            psi_1(i,j)=gamma_1(1)+gamma_1(2)*c1(i,j)+gamma_1(3)*c2(i,j)+gamma_1(4)*c3(i,j) &
                                & +gamma_1(5)*abs(Nu(i)-Nu(j))+gamma_1(6)*abs(Co(i)-Co(j))+gamma_1(7)*abs(Ex(i)-Ex(j))-gamma_1(8)*abs(zz_1(i,1)-zz_1(j,1)) &
                                & -gamma_1(9)*abs(zz_1(i,2)-zz_1(j,2))
                            psi_2(i,j)=gamma_2(1)+gamma_2(2)*c1(i,j)+gamma_2(3)*c2(i,j)+gamma_2(4)*c3(i,j) &
                                & +gamma_2(5)*abs(Nu(i)-Nu(j))+gamma_2(6)*abs(Co(i)-Co(j))+gamma_2(7)*abs(Ex(i)-Ex(j))-gamma_2(8)*abs(zz_1(i,1)-zz_1(j,1)) &
                                & -gamma_2(9)*abs(zz_1(i,2)-zz_1(j,2))
                        ELSEIF (DD==3) THEN
                            psi_1(i,j)=gamma_1(1)+gamma_1(2)*c1(i,j)+gamma_1(3)*c2(i,j)+gamma_1(4)*c3(i,j) &
                                & +gamma_1(5)*abs(Nu(i)-Nu(j))+gamma_1(6)*abs(Co(i)-Co(j))+gamma_1(7)*abs(Ex(i)-Ex(j))-gamma_1(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_1(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                &-gamma_1(10)*abs(zz_1(i,3)-zz_1(j,3))
                            psi_2(i,j)=gamma_2(1)+gamma_2(2)*c1(i,j)+gamma_2(3)*c2(i,j)+gamma_2(4)*c3(i,j) &
                                & +gamma_2(5)*abs(Nu(i)-Nu(j))+gamma_2(6)*abs(Co(i)-Co(j))+gamma_2(7)*abs(Ex(i)-Ex(j))-gamma_2(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_2(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                &-gamma_2(10)*abs(zz_1(i,3)-zz_1(j,3))
                        ELSEIF (DD==4) THEN
                            psi_1(i,j)=gamma_1(1)+gamma_1(2)*c1(i,j)+gamma_1(3)*c2(i,j)+gamma_1(4)*c3(i,j)&
                                & +gamma_1(5)*abs(Nu(i)-Nu(j))+gamma_1(6)*abs(Co(i)-Co(j))+gamma_1(7)*abs(Ex(i)-Ex(j))-gamma_1(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_1(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_1(10)*abs(zz_1(i,3)-zz_1(j,3))-gamma_1(11)*abs(zz_1(i,4)-zz_1(j,4))
                            psi_2(i,j)=gamma_2(1)+gamma_2(2)*c1(i,j)+gamma_2(3)*c2(i,j)+gamma_2(4)*c3(i,j)&
                                & +gamma_2(5)*abs(Nu(i)-Nu(j))+gamma_2(6)*abs(Co(i)-Co(j))+gamma_2(7)*abs(Ex(i)-Ex(j))-gamma_2(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_2(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_2(10)*abs(zz_1(i,3)-zz_1(j,3))-gamma_2(11)*abs(zz_1(i,4)-zz_1(j,4))
                        ELSEIF (DD==5) THEN
                            psi_1(i,j)=gamma_1(1)+gamma_1(2)*c1(i,j)+gamma_1(3)*c2(i,j)+gamma_1(4)*c3(i,j)&
                                & +gamma_1(5)*abs(Nu(i)-Nu(j))+gamma_1(6)*abs(Co(i)-Co(j))+gamma_1(7)*abs(Ex(i)-Ex(j))-gamma_1(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_1(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_1(10)*abs(zz_1(i,3)-zz_1(j,3))-gamma_1(11)*abs(zz_1(i,4)-zz_1(j,4))-gamma_1(12)*abs(zz_1(i,5)-zz_1(j,5))
                            psi_2(i,j)=gamma_2(1)+gamma_2(2)*c1(i,j)+gamma_2(3)*c2(i,j)+gamma_2(4)*c3(i,j)&
                                & +gamma_2(5)*abs(Nu(i)-Nu(j))+gamma_2(6)*abs(Co(i)-Co(j))+gamma_2(7)*abs(Ex(i)-Ex(j))-gamma_2(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_2(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_2(10)*abs(zz_1(i,3)-zz_1(j,3))-gamma_2(11)*abs(zz_1(i,4)-zz_1(j,4))-gamma_2(12)*abs(zz_1(i,5)-zz_1(j,5))
                        ELSEIF (DD==6) THEN
                            psi_1(i,j)=gamma_1(1)+gamma_1(2)*c1(i,j)+gamma_1(3)*c2(i,j)+gamma_1(4)*c3(i,j)&
                                & +gamma_1(5)*abs(Nu(i)-Nu(j))+gamma_1(6)*abs(Co(i)-Co(j))+gamma_1(7)*abs(Ex(i)-Ex(j))-gamma_1(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_1(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_1(10)*abs(zz_1(i,3)-zz_1(j,3))-gamma_1(11)*abs(zz_1(i,4)-zz_1(j,4))-gamma_1(12)*abs(zz_1(i,5)-zz_1(j,5)) -gamma_1(13)*abs(zz_1(i,6)-zz_1(j,6))
                            psi_2(i,j)=gamma_2(1)+gamma_2(2)*c1(i,j)+gamma_2(3)*c2(i,j)+gamma_2(4)*c3(i,j)&
                                & +gamma_2(5)*abs(Nu(i)-Nu(j))+gamma_2(6)*abs(Co(i)-Co(j))+gamma_2(7)*abs(Ex(i)-Ex(j))-gamma_2(8)*abs(zz_1(i,1)-zz_1(j,1))-gamma_2(9)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_2(10)*abs(zz_1(i,3)-zz_1(j,3))-gamma_2(11)*abs(zz_1(i,4)-zz_1(j,4))-gamma_2(12)*abs(zz_1(i,5)-zz_1(j,5)) -gamma_2(13)*abs(zz_1(i,6)-zz_1(j,6))
                        ENDIF
                        q_1=psi_1(i,j)*W(i,j)-log(1+exp(psi_1(i,j)))
                        q_2=psi_2(i,j)*W(i,j)-log(1+exp(psi_2(i,j)))
                        pp=pp+(q_1-q_2)
                    ENDIF
                END DO
            END DO

            pp_G(g)=pp

            ! log likelihood from outcome equation !

            S1=eye(NN)-lambda_1*w_norm(1:NN,1:NN,g)
            S2=eye(NN)-lambda_T(t-1)*w_norm(1:NN,1:NN,g)


            XX1(:,31:30+DD)=zz_1
            XX1(:,31+DD:30+2*DD)=matmul(w_norm(1:NN,1:NN,g),zz_1)

            ep_1=matmul(S1,Y)-matmul(XX1,beta_yT(:,t-1))-alpha_yT(g,t-1)
            ep_2=matmul(S2,Y)-matmul(XX1,beta_yT(:,t-1))-alpha_yT(g,t-1)


            like_Y1=log(FindDet(S1, NN))-dot_product(ep_1, ep_1)/(2*sigma_eT(t-1))
            like_Y2=log(FindDet(S2, NN))-dot_product(ep_2, ep_2)/(2*sigma_eT(t-1))

            pp_l(g)=(like_Y1-like_Y2)


            DEALLOCATE(S);
            DEALLOCATE(S1);       DEALLOCATE(S2);
            DEALLOCATE(Vl);       DEALLOCATE(Vn);
            DEALLOCATE(Vc);       DEALLOCATE(Ve);
            DEALLOCATE(Y);
            DEALLOCATE(Nu);
            DEALLOCATE(Co);
            DEALLOCATE(Ex);
            DEALLOCATE(X1) ;       DEALLOCATE(XX1)
            DEALLOCATE(X2) ;       DEALLOCATE(XX2)
            DEALLOCATE(W);
            DEALLOCATE(psi_1);    DEALLOCATE(psi_2)
            DEALLOCATE(ep);
            DEALLOCATE(ep_1);     DEALLOCATE(ep_2)
            DEALLOCATE(ep_1v);    DEALLOCATE(ep_2v)
            DEALLOCATE(ep_n);
            DEALLOCATE(ep_n1);     DEALLOCATE(ep_n2)
            DEALLOCATE(ep_n1v);    DEALLOCATE(ep_n2v)
            DEALLOCATE(ep_c);
            DEALLOCATE(ep_c1);     DEALLOCATE(ep_c2)
            DEALLOCATE(ep_c1v);    DEALLOCATE(ep_c2v)
            DEALLOCATE(ep_e);
            DEALLOCATE(ep_e1);     DEALLOCATE(ep_e2)
            DEALLOCATE(ep_e1v);    DEALLOCATE(ep_e2v)
            DEALLOCATE(C1);       DEALLOCATE(C2);
            DEALLOCATE(C3);
            DEALLOCATE(V1);       DEALLOCATE(V2)
            DEALLOCATE(zz_1);     DEALLOCATE(zz_2)
            DEALLOCATE(IPIV);     DEALLOCATE(INDX)
            DEALLOCATE(IPVT);     DEALLOCATE(FACT)
        END DO
        !$omp end parallel

        pp=sum(pp_G)+logmvnpdf(gamma_1, G_0, 7+DD)-logmvnpdf(gamma_2, G_0, 7+DD)

        CALL random_number(rand)
        IF (log(rand)<=pp) THEN
            gamma_T(:,t)=gamma_1
            acc_1=acc_1+1.0
        ELSE
            gamma_T(:,t)=gamma_T(:,t-1)
        ENDIF
        acc_rate1(t)=acc_1/t

        CALL random_number(rand)
        IF (log(rand)<=sum(pp_l)) THEN
            lambda_T(t)=lambda_1
            acc_2=acc_2+1.0
        ELSE
            lambda_T(t)=lambda_T(t-1)
        END IF
        acc_rate2(t)=acc_2/t



        !***************GIBBS STEP TO SIMULTE BETA-outcome *********************!

        ALLOCATE(XVX(30+2*DD,30+2*DD,GG))
        ALLOCATE(XVY(30+2*DD,GG))
        ALLOCATE(INV_B1(30+2*DD,30+2*DD))
        ALLOCATE(INV_B2(30+2*DD,30+2*DD))
        ALLOCATE(beta_temp(30+2*DD))
        XVX=0.0
        XVY=0.0

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Vl, Y, YY, &
        !$omp & X1, XX1, XX2, S1, V1, V2, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Vl(NN,NN))
            ALLOCATE(Y(NN));
            ALLOCATE(YY(NN))
            ALLOCATE(X1(NN,15)) ;
            ALLOCATE(XX1(NN,30+2*DD));
            ALLOCATE(XX2(NN,30+2*DD)) ;
            ALLOCATE(S1(NN,NN));
            ALLOCATE(V1(NN,NN));
            ALLOCATE(V2(NN,NN))
            ALLOCATE(zz_1(NN,DD))
            ALLOCATE(IPIV(NN))

            Vl=sigma_eT(t-1)*EYE(NN)
            Y=group(1:NN,dep,g)
            X1=group(1:NN,4:18,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:15)=X1
            XX1(:,16:30)=matmul(w_norm(1:NN,1:NN,g),X1)
            XX1(:,31:30+DD)=zz_1
            XX1(:,31+DD:30+2*DD)=matmul(w_norm(1:NN,1:NN,g),zz_1)
            S1=eye(NN)-lambda_T(t)*w_norm(1:NN,1:NN,g)

            XX2=XX1
            V1=Vl
            CALL SGESV(NN,30+2*DD,V1,NN,IPIV,XX2,NN,INFO)
            YY=matmul(S1,Y)-alpha_yT(g,t-1)
            V2=Vl
            CALL SGESV(NN,1,V2,NN,IPIV,YY,NN,INFO)


            XVX(:,:,g)=matmul(transpose(XX1) , XX2)
            XVY(:,g)=matmul(transpose(XX1) , YY)

            DEALLOCATE(Vl)
            DEALLOCATE(Y);    DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1) ; DEALLOCATE(XX2) ;
            DEALLOCATE(S1);
            DEALLOCATE(V1); DEALLOCATE(V2)
            DEALLOCATE(zz_1)
            DEALLOCATE(IPIV)
        END DO
        !$omp end parallel

        CALL FINDInv(sum(XVX,3)+INV_B_10,INV_B1,30+2*DD,ier)
        INV_B2=INV_B1
        CALL spotrf('U',30+2*DD, INV_B2, 30+2*DD, info)
        errcode=vsrnggaussianmv(method, stream, 1, beta_temp, 30+2*DD, me, beta_10,INV_B2)
        beta_yT(:,t)=beta_temp+matmul(INV_B1,sum(XVY,2))

        DEALLOCATE(XVX)
        DEALLOCATE(XVY)
        DEALLOCATE(INV_B1)
        DEALLOCATE(INV_B2)
        DEALLOCATE(beta_temp)


        !***************GIBBS STEP TO SIMULTE BETA_Neuroticism *********************!

        ALLOCATE(XVX(10+DD,10+DD,GG))
        ALLOCATE(XVY(10+DD,GG))
        ALLOCATE(INV_B1(10+DD,10+DD))
        ALLOCATE(INV_B2(10+DD,10+DD))
        ALLOCATE(beta_temp(10+DD))
        XVX=0.0
        XVY=0.0

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Vn, Y, YY, &
        !$omp & X1, XX1, XX2, W, V1, V2, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Vn(NN,NN))
            ALLOCATE(Y(NN));        ALLOCATE(YY(NN))
            ALLOCATE(X1(NN,10)) ;
            ALLOCATE(XX1(NN,10+DD));
            ALLOCATE(XX2(NN,10+DD));
            ALLOCATE(W(NN,NN));
            ALLOCATE(V1(NN,NN)); ALLOCATE(V2(NN,NN))
            ALLOCATE(zz_1(NN,DD))
            ALLOCATE(IPIV(NN))

            Vn=sigma_v1T(t-1)*EYE(NN)
            Y=group(1:NN,4,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)

            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            XX2=XX1
            V1=Vn
            CALL SGESV(NN,10+DD,V1,NN,IPIV,XX2,NN,INFO)
            YY=Y-alpha_nT(g,t-1)
            V2=Vn
            CALL SGESV(NN,1,V2,NN,IPIV,YY,NN,INFO)

            XVX(:,:,g)=matmul(transpose(XX1) , XX2)
            XVY(:,g)=matmul(transpose(XX1) , YY)

            DEALLOCATE(Vn)
            DEALLOCATE(Y);    DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1)
            DEALLOCATE(XX2)
            DEALLOCATE(W);
            DEALLOCATE(V1); DEALLOCATE(V2)
            DEALLOCATE(zz_1)
            DEALLOCATE(IPIV)
        END DO
        !$omp end parallel

        CALL FINDInv(sum(XVX,3)+INV_B_20,INV_B1,10+DD,ier)
        INV_B2=INV_B1
        CALL spotrf('U',10+DD, INV_B2, 10+DD, info)
        errcode=vsrnggaussianmv(method, stream, 1, beta_temp, 10+DD, me, beta_20,INV_B2)
        beta_nT(:,t)=beta_temp+matmul(INV_B1,sum(XVY,2))

        DEALLOCATE(XVX)
        DEALLOCATE(XVY)
        DEALLOCATE(INV_B1)
        DEALLOCATE(INV_B2)
        DEALLOCATE(beta_temp)


        !***************GIBBS STEP TO SIMULTE BETA-Conscientiuouness *********************!

        ALLOCATE(XVX(10+DD,10+DD,GG))
        ALLOCATE(XVY(10+DD,GG))
        ALLOCATE(INV_B1(10+DD,10+DD))
        ALLOCATE(INV_B2(10+DD,10+DD))
        ALLOCATE(beta_temp(10+DD))
        XVX=0.0
        XVY=0.0

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Vn, Y, YY, &
        !$omp & X1, XX1, XX2, W, V1, V2, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Vn(NN,NN))
            ALLOCATE(Y(NN));        ALLOCATE(YY(NN))
            ALLOCATE(X1(NN,10)) ;
            ALLOCATE(XX1(NN,10+DD));
            ALLOCATE(XX2(NN,10+DD));
            ALLOCATE(W(NN,NN));
            ALLOCATE(V1(NN,NN)); ALLOCATE(V2(NN,NN))
            ALLOCATE(zz_1(NN,DD))
            ALLOCATE(IPIV(NN))

            Vn=sigma_v2T(t-1)*EYE(NN)
            Y=group(1:NN,5,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)

            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            XX2=XX1
            V1=Vn
            CALL SGESV(NN,10+DD,V1,NN,IPIV,XX2,NN,INFO)
            YY=Y-alpha_cT(g,t-1)
            V2=Vn
            CALL SGESV(NN,1,V2,NN,IPIV,YY,NN,INFO)

            XVX(:,:,g)=matmul(transpose(XX1) , XX2)
            XVY(:,g)=matmul(transpose(XX1) , YY)

            DEALLOCATE(Vn)
            DEALLOCATE(Y);    DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1)
            DEALLOCATE(XX2)
            DEALLOCATE(W);
            DEALLOCATE(V1); DEALLOCATE(V2)
            DEALLOCATE(zz_1)
            DEALLOCATE(IPIV)
        END DO
        !$omp end parallel

        CALL FINDInv(sum(XVX,3)+INV_B_20,INV_B1,10+DD,ier)
        INV_B2=INV_B1
        CALL spotrf('U',10+DD, INV_B2, 10+DD, info)
        errcode=vsrnggaussianmv(method, stream, 1, beta_temp, 10+DD, me, beta_20,INV_B2)
        beta_cT(:,t)=beta_temp+matmul(INV_B1,sum(XVY,2))

        DEALLOCATE(XVX)
        DEALLOCATE(XVY)
        DEALLOCATE(INV_B1)
        DEALLOCATE(INV_B2)
        DEALLOCATE(beta_temp)

        !***************GIBBS STEP TO SIMULTE BETA-extraversion *********************!

        ALLOCATE(XVX(10+DD,10+DD,GG))
        ALLOCATE(XVY(10+DD,GG))
        ALLOCATE(INV_B1(10+DD,10+DD))
        ALLOCATE(INV_B2(10+DD,10+DD))
        ALLOCATE(beta_temp(10+DD))
        XVX=0.0
        XVY=0.0

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Vn, Y, YY, &
        !$omp & X1, XX1, XX2, W, V1, V2, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Vn(NN,NN))
            ALLOCATE(Y(NN));        ALLOCATE(YY(NN))
            ALLOCATE(X1(NN,10)) ;
            ALLOCATE(XX1(NN,10+DD));
            ALLOCATE(XX2(NN,10+DD));
            ALLOCATE(W(NN,NN));
            ALLOCATE(V1(NN,NN)); ALLOCATE(V2(NN,NN))
            ALLOCATE(zz_1(NN,DD))
            ALLOCATE(IPIV(NN))

            Vn=sigma_v3T(t-1)*EYE(NN)
            Y=group(1:NN,6,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            XX2=XX1
            V1=Vn
            CALL SGESV(NN,10+DD,V1,NN,IPIV,XX2,NN,INFO)
            YY=Y-alpha_eT(g,t-1)
            V2=Vn
            CALL SGESV(NN,1,V2,NN,IPIV,YY,NN,INFO)

            XVX(:,:,g)=matmul(transpose(XX1) , XX2)
            XVY(:,g)=matmul(transpose(XX1) , YY)

            DEALLOCATE(Vn)
            DEALLOCATE(Y);    DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1)
            DEALLOCATE(XX2)
            DEALLOCATE(W);
            DEALLOCATE(V1); DEALLOCATE(V2)
            DEALLOCATE(zz_1)
            DEALLOCATE(IPIV)
        END DO
        !$omp end parallel

        CALL FINDInv(sum(XVX,3)+INV_B_20,INV_B1,10+DD,ier)
        INV_B2=INV_B1
        CALL spotrf('U',10+DD, INV_B2, 10+DD, info)
        errcode=vsrnggaussianmv(method, stream, 1, beta_temp, 10+DD, me, beta_20,INV_B2)
        beta_eT(:,t)=beta_temp+matmul(INV_B1,sum(XVY,2))

        DEALLOCATE(XVX)
        DEALLOCATE(XVY)
        DEALLOCATE(INV_B1)
        DEALLOCATE(INV_B2)
        DEALLOCATE(beta_temp)



        !******************M-H STEP FOR SIGMA_E *****************!

        rho_1=sum(N)

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y,  &
        !$omp & X1, XX1, W, S, ep, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend
            NN=N(G)
            ALLOCATE(Y(NN))
            ALLOCATE(X1(NN,15))
            ALLOCATE(XX1(NN,30+2*DD))
            ALLOCATE(W(NN,NN))
            ALLOCATE(S(NN,NN))
            ALLOCATE(ep(NN))
            ALLOCATE(zz_1(NN,DD))

            Y=group(1:NN,dep,g)
            X1=group(1:NN,4:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:15)=X1
            XX1(:,16:30)=matmul(w_norm(1:NN,1:NN,g),X1)
            XX1(:,31:30+DD)=zz_1
            XX1(:,31+DD:30+2*DD)=matmul(w_norm(1:NN,1:NN,g),zz_1)

            S=eye(NN)-lambda_T(t)*w_norm(1:NN,1:NN,g)

            ep=matmul(S,Y)-matmul(XX1,beta_yT(:,t))-alpha_yT(g,t-1)
            ep_v(g)=dot_product(ep,ep)
            DEALLOCATE(Y)
            DEALLOCATE(X1)
            DEALLOCATE(XX1)
            DEALLOCATE(W)
            DEALLOCATE(S)
            DEALLOCATE(ep)
            DEALLOCATE(zz_1)
        END DO
        !$omp end parallel
        sigma_eT(t)=(sum(ep_v)+eta_0)/random_chisq(rho_1+rho_0, first)


        !******************M-H STEP FOR SIGMA_V1  *****************!

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y,  &
        !$omp & X1, XX1, W, ep, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend
            NN=N(G)
            ALLOCATE(Y(NN))
            ALLOCATE(X1(NN,10))
            ALLOCATE(XX1(NN,10+DD))
            ALLOCATE(W(NN,NN))
            ALLOCATE(ep(NN))
            ALLOCATE(zz_1(NN,DD))

            Y=group(1:NN,4,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            ep=Y-matmul(XX1,beta_nT(:,t))-alpha_nT(g,t-1)
            ep_v(g)=dot_product(ep,ep)
            DEALLOCATE(Y)
            DEALLOCATE(X1)
            DEALLOCATE(XX1)
            DEALLOCATE(W)
            DEALLOCATE(ep)
            DEALLOCATE(zz_1)
        END DO
        !$omp end parallel
        sigma_v1T(t)=(sum(ep_v)+eta_0)/random_chisq(rho_1+rho_0, first)



        !******************M-H STEP FOR SIGMA_V2 *****************!

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y,  &
        !$omp & X1, XX1, W, ep, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend
            NN=N(G)
            ALLOCATE(Y(NN))
            ALLOCATE(X1(NN,10))
            ALLOCATE(XX1(NN,10+DD))
            ALLOCATE(W(NN,NN))
            ALLOCATE(ep(NN))
            ALLOCATE(zz_1(NN,DD))

            Y=group(1:NN,5,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            ep=Y-matmul(XX1,beta_cT(:,t))-alpha_cT(g,t-1)
            ep_v(g)=dot_product(ep,ep)
            DEALLOCATE(Y)
            DEALLOCATE(X1)
            DEALLOCATE(XX1)
            DEALLOCATE(W)
            DEALLOCATE(ep)
            DEALLOCATE(zz_1)
        END DO
        !$omp end parallel
        sigma_v2T(t)=(sum(ep_v)+eta_0)/random_chisq(rho_1+rho_0, first)



        !******************M-H STEP FOR SIGMA_V3 *****************!

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y,  &
        !$omp & X1, XX1, W, ep, zz_1, IPIV, INFO, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend
            NN=N(G)
            ALLOCATE(Y(NN))
            ALLOCATE(X1(NN,10))
            ALLOCATE(XX1(NN,10+DD))
            ALLOCATE(W(NN,NN))
            ALLOCATE(ep(NN))
            ALLOCATE(zz_1(NN,DD))

            Y=group(1:NN,6,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            ep=Y-matmul(XX1,beta_eT(:,t))-alpha_eT(g,t-1)
            ep_v(g)=dot_product(ep,ep)
            DEALLOCATE(Y)
            DEALLOCATE(X1)
            DEALLOCATE(XX1)
            DEALLOCATE(W)
            DEALLOCATE(ep)
            DEALLOCATE(zz_1)
        END DO
        !$omp end parallel
        sigma_v3T(t)=(sum(ep_v)+eta_0)/random_chisq(rho_1+rho_0, first)





        !*************GIBBS STEP FOR ALPHA1_G*********************!


        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y, YY, &
        !$omp & X1, XX1, W, S1, zz_1, IPIV, R_g, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Y(NN));
            ALLOCATE(YY(NN));
            ALLOCATE(X1(NN,15)) ;
            ALLOCATE(XX1(NN,30+2*DD)) ;
            ALLOCATE(W(NN,NN));
            ALLOCATE(S1(NN,NN));
            ALLOCATE(zz_1(NN,DD))

            R_g=(ALPHA_0**(-1)+sigma_eT(t)**(-1.0)*NN)**(-1.0)
            Y=group(1:NN,dep,g)
            X1=group(1:NN,4:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:15)=X1
            XX1(:,16:30)=matmul(w_norm(1:NN,1:NN,g),X1)
            XX1(:,31:30+DD)=zz_1
            XX1(:,31+DD:30+2*DD)=matmul(w_norm(1:NN,1:NN,g),zz_1)

            S1=eye(NN)-lambda_T(t)*w_norm(1:NN,1:NN,g)

            YY=matmul(S1,Y)
            alpha_yT(g,t)=R_g*(ALPHA_0**(-1)*a_0+sigma_eT(t)**(-1.0)*sum((YY-matmul(XX1,beta_yT(:,t)))))+random_normal()*sqrt(R_g)

            DEALLOCATE(Y);
            DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1) ;
            DEALLOCATE(W);
            DEALLOCATE(S1);
            DEALLOCATE(zz_1);
        END DO
        !$omp end parallel



        !*************GIBBS STEP FOR ALPHA2_G*********************!


        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y, YY, &
        !$omp & X1, XX1, W, zz_1, IPIV, R_g, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Y(NN));
            ALLOCATE(YY(NN));
            ALLOCATE(X1(NN,10)) ;
            ALLOCATE(XX1(NN,10+DD)) ;
            ALLOCATE(W(NN,NN));
            ALLOCATE(zz_1(NN,DD))

            R_g=(ALPHA_0**(-1)+sigma_v1T(t)**(-1.0)*NN)**(-1.0)
            Y=group(1:NN,4,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            YY=Y
            alpha_nT(g,t)=R_g*(ALPHA_0**(-1)*a_0+sigma_v1T(t)**(-1.0)*sum((YY-matmul(XX1,beta_nT(:,t)))))+random_normal()*sqrt(R_g)

            DEALLOCATE(Y);
            DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1) ;
            DEALLOCATE(W);
            DEALLOCATE(zz_1);
        END DO
        !$omp end parallel


        !*************GIBBS STEP FOR ALPHA3_G*********************!


        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y, YY, &
        !$omp & X1, XX1, W, S1, zz_1, IPIV, R_g, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Y(NN));
            ALLOCATE(YY(NN));
            ALLOCATE(X1(NN,10)) ;
            ALLOCATE(XX1(NN,10+DD)) ;
            ALLOCATE(W(NN,NN));
            ALLOCATE(zz_1(NN,DD))

            R_g=(ALPHA_0**(-1)+sigma_v2T(t)**(-1.0)*NN)**(-1.0)
            Y=group(1:NN,5,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            YY=Y
            alpha_cT(g,t)=R_g*(ALPHA_0**(-1)*a_0+sigma_v2T(t)**(-1.0)*sum((YY-matmul(XX1,beta_cT(:,t)))))+random_normal()*sqrt(R_g)

            DEALLOCATE(Y);
            DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1) ;
            DEALLOCATE(W);
            DEALLOCATE(zz_1);
        END DO
        !$omp end parallel


        !*************GIBBS STEP FOR ALPHA4_G*********************!


        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g,NN, Y, YY, &
        !$omp & X1, XX1, W,  zz_1, IPIV, R_g, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Y(NN));
            ALLOCATE(YY(NN));
            ALLOCATE(X1(NN,10)) ;
            ALLOCATE(XX1(NN,10+DD)) ;
            ALLOCATE(W(NN,NN));
            ALLOCATE(zz_1(NN,DD))

            R_g=(ALPHA_0**(-1)+sigma_v3T(t)**(-1.0)*NN)**(-1.0)
            Y=group(1:NN,6,g)
            X1=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)
            XX1(:,1:10)=X1
            XX1(:,11:10+DD)=zz_1

            YY=Y
            alpha_eT(g,t)=R_g*(ALPHA_0**(-1)*a_0+sigma_v3T(t)**(-1.0)*sum((YY-matmul(XX1,beta_eT(:,t)))))+random_normal()*sqrt(R_g)

            DEALLOCATE(Y);
            DEALLOCATE(YY)
            DEALLOCATE(X1) ;
            DEALLOCATE(XX1) ;
            DEALLOCATE(W);
            DEALLOCATE(zz_1);
        END DO
        !$omp end parallel


        !******** Calculate Log likelihood function value *********!

        Call OMP_SET_NUM_THREADS(Nthreads)
        !$omp parallel default(shared) private(g, NN, Y, Nu, Co, Ex, X1, XX1, X2, XX2, W, &
        !$omp & S, psi_1, ep_1, ep_1v, INFO, CODE, IPVT, FACT, &
        !$omp & ep_n1, ep_n1v, ep_c1, ep_c1v, ep_e1, ep_e1v, &
        !$omp & C1, C2, C3, Vl, Vn, Vc, Ve, V1, zz_1, IPIV, INDX, DET1, DET2, q_1, &
        !$omp & like_Y1, like_N1, like_C1, like_E1, &
        !$omp & pp, i, j, rand, &
        !$omp & ithread, iblock, ibegin, iend)

        ithread = OMP_GET_THREAD_NUM()
        iblock = (GG+Nthreads-1)/Nthreads
        ibegin = ithread*iblock+1
        iend = min((ithread+1)*iblock,GG)

        DO g = ibegin,iend

            NN=N(g)
            ALLOCATE(Vl(NN,NN));       ALLOCATE(Vn(NN,NN));
            ALLOCATE(Vc(NN,NN));       ALLOCATE(Ve(NN,NN));
            ALLOCATE(V1(NN,NN));       ALLOCATE(S(NN,NN));
            ALLOCATE(Y(NN));           ALLOCATE(Nu(NN));
            ALLOCATE(Co(NN));          ALLOCATE(Ex(NN));
            ALLOCATE(X1(NN,15)) ;      ALLOCATE(XX1(NN,30+2*DD)) ;
            ALLOCATE(X2(NN,10)) ;      ALLOCATE(XX2(NN,10+DD)) ;
            ALLOCATE(W(NN,NN));
            ALLOCATE(psi_1(NN,NN));
            ALLOCATE(ep_1(NN));
            ALLOCATE(ep_1v(NN));
            ALLOCATE(ep_n1(NN));        ALLOCATE(ep_n1v(NN));
            ALLOCATE(ep_c1(NN));        ALLOCATE(ep_c1v(NN));
            ALLOCATE(ep_e1(NN));        ALLOCATE(ep_e1v(NN));
            ALLOCATE(C1(NN,NN));       ALLOCATE(C2(NN,NN));
            ALLOCATE(C3(NN,NN));
            ALLOCATE(zz_1(NN,DD));
            ALLOCATE(IPIV(NN));        ALLOCATE(INDX(NN));
            ALLOCATE(IPVT(NN));        ALLOCATE(FACT(NN,NN));

            Vl=sigma_eT(t)*EYE(NN)
            Vn=sigma_v1T(t)*EYE(NN)
            Vc=sigma_v2T(t)*EYE(NN)
            Ve=sigma_v3T(t)*EYE(NN)
            Y=group(1:NN,dep,g)
            Nu=group(1:NN,4,g)
            Co=group(1:NN,5,g)
            Ex=group(1:NN,6,g)
            X1=group(1:NN,4:18,g)
            X2=group(1:NN,9:18,g)
            W=network(1:NN,1:NN,g)
            zz_1=zz(1:NN,1:DD,g)

            XX1(:,1:15)=X1
            XX1(:,16:30)=matmul(w_norm(1:NN,1:NN,g),X1)
            XX1(:,31:30+DD)=zz_1
            XX1(:,31+DD:30+2*DD)=matmul(w_norm(1:NN,1:NN,g),zz_1)

            XX2(:,1:10)=X2
            XX2(:,11:10+DD)=zz_1

            C1=grade(1:NN,1:NN,g)
            C2=sex(1:NN,1:NN,g)
            C3=race(1:NN,1:NN,g)

            pp=0.0
            DO i=1,NN
                DO j=1, NN
                    IF (j/=i) THEN
                        IF (DD==1) THEN
                            psi_1(i,j)=gamma_T(1,t)+gamma_T(2,t)*c1(i,j)+gamma_T(3,t)*c2(i,j)+gamma_T(4,t)*c3(i,j) &
                                & +gamma_T(5,t)*abs(Nu(i)-Nu(j))+gamma_T(6,t)*abs(Co(i)-Co(j))+gamma_T(7,t)*abs(Ex(i)-Ex(j)) &
                                & -gamma_T(8,t)*abs(zz_1(i,1)-zz_1(j,1))
                        ELSEIF (DD==2) THEN
                            psi_1(i,j)=gamma_T(1,t)+gamma_T(2,t)*c1(i,j)+gamma_T(3,t)*c2(i,j)+gamma_T(4,t)*c3(i,j)&
                                & +gamma_T(5,t)*abs(Nu(i)-Nu(j))+gamma_T(6,t)*abs(Co(i)-Co(j))+gamma_T(7,t)*abs(Ex(i)-Ex(j)) &
                                & -gamma_T(8,t)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t)*abs(zz_1(i,2)-zz_1(j,2))
                        ELSEIF (DD==3) THEN
                            psi_1(i,j)=gamma_T(1,t)+gamma_T(2,t)*c1(i,j)+gamma_T(3,t)*c2(i,j)+gamma_T(4,t)*c3(i,j)&
                                & +gamma_T(5,t)*abs(Nu(i)-Nu(j))+gamma_T(6,t)*abs(Co(i)-Co(j))+gamma_T(7,t)*abs(Ex(i)-Ex(j)) &
                                & -gamma_T(8,t)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_T(10,t)*abs(zz_1(i,3)-zz_1(j,3))
                        ELSEIF (DD==4) THEN
                            psi_1(i,j)=gamma_T(1,t)+gamma_T(2,t)*c1(i,j)+gamma_T(3,t)*c2(i,j)+gamma_T(4,t)*c3(i,j)&
                                & +gamma_T(5,t)*abs(Nu(i)-Nu(j))+gamma_T(6,t)*abs(Co(i)-Co(j))+gamma_T(7,t)*abs(Ex(i)-Ex(j)) &
                                & -gamma_T(8,t)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_T(10,t)*abs(zz_1(i,3)-zz_1(j,3)) -gamma_T(11,t)*abs(zz_1(i,4)-zz_1(j,4))
                        ELSEIF (DD==5) THEN
                            psi_1(i,j)=gamma_T(1,t)+gamma_T(2,t)*c1(i,j)+gamma_T(3,t)*c2(i,j)+gamma_T(4,t)*c3(i,j)&
                                & +gamma_T(5,t)*abs(Nu(i)-Nu(j))+gamma_T(6,t)*abs(Co(i)-Co(j))+gamma_T(7,t)*abs(Ex(i)-Ex(j)) &
                                & -gamma_T(8,t)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_T(10,t)*abs(zz_1(i,3)-zz_1(j,3)) -gamma_T(11,t)*abs(zz_1(i,4)-zz_1(j,4)) -gamma_T(12,t)*abs(zz_1(i,5)-zz_1(j,5))
                        ELSEIF (DD==6) THEN
                            psi_1(i,j)=gamma_T(1,t)+gamma_T(2,t)*c1(i,j)+gamma_T(3,t)*c2(i,j)+gamma_T(4,t)*c3(i,j) &
                                & +gamma_T(5,t)*abs(Nu(i)-Nu(j))+gamma_T(6,t)*abs(Co(i)-Co(j))+gamma_T(7,t)*abs(Ex(i)-Ex(j)) &
                                & -gamma_T(8,t)*abs(zz_1(i,1)-zz_1(j,1))-gamma_T(9,t)*abs(zz_1(i,2)-zz_1(j,2)) &
                                & -gamma_T(10,t)*abs(zz_1(i,3)-zz_1(j,3)) -gamma_T(11,t)*abs(zz_1(i,4)-zz_1(j,4)) -gamma_T(12,t)*abs(zz_1(i,5)-zz_1(j,5)) &
                                & -gamma_T(13,t)*abs(zz_1(i,6)-zz_1(j,6))
                        ENDIF
                        q_1=psi_1(i,j)*W(i,j)-log(1+exp(psi_1(i,j)))
                        pp=pp+q_1
                    ENDIF
                END DO
            END DO

            S=eye(NN)-lambda_T(t)*w_norm(1:NN,1:NN,g)

            ep_1=matmul(S,Y)-matmul(XX1,beta_yT(:,t))-alpha_yT(g,t)
            det1=FindDet(S, NN)
            like_Y1=-0.5*NN*log(2*pi)-0.5*NN*log(sigma_eT(t))+log(det1)-dot_product(ep_1, ep_1)/(2*sigma_eT(t))

            ep_n1=Nu-matmul(XX2,beta_nT(:,t))-alpha_nT(g,t)
            like_N1=-0.5*NN*log(2*pi)-0.5*NN*log(sigma_v1T(t))-dot_product(ep_n1, ep_n1)/(2*sigma_v1T(t))


            ep_c1=Co-matmul(XX2,beta_cT(:,t))-alpha_cT(g,t)
            like_C1=-0.5*NN*log(2*pi)-0.5*NN*log(sigma_v2T(t))-dot_product(ep_c1, ep_c1)/(2*sigma_v2T(t))

            ep_e1=Ex-matmul(XX2,beta_eT(:,t))-alpha_eT(g,t)
            like_E1=-0.5*NN*log(2*pi)-0.5*NN*log(sigma_v3T(t))-dot_product(ep_e1, ep_e1)/(2*sigma_v3T(t))

            pp_G(g)=pp+like_Y1+like_N1+like_C1+like_E1

            DEALLOCATE(Vl);       DEALLOCATE(Vn);
            DEALLOCATE(Vc);       DEALLOCATE(Ve);
            DEALLOCATE(V1);       DEALLOCATE(S);
            DEALLOCATE(Y);        DEALLOCATE(Nu);
            DEALLOCATE(Co);       DEALLOCATE(Ex);
            DEALLOCATE(X1) ;      DEALLOCATE(XX1) ;
            DEALLOCATE(X2) ;      DEALLOCATE(XX2) ;
            DEALLOCATE(W);
            DEALLOCATE(psi_1);
            DEALLOCATE(ep_1);
            DEALLOCATE(ep_1v);
            DEALLOCATE(ep_n1);    DEALLOCATE(ep_n1v);
            DEALLOCATE(ep_c1);    DEALLOCATE(ep_c1v);
            DEALLOCATE(ep_e1);    DEALLOCATE(ep_e1v);
            DEALLOCATE(C1);       DEALLOCATE(C2);
            DEALLOCATE(C3);
            DEALLOCATE(zz_1);
            DEALLOCATE(IPIV);     DEALLOCATE(INDX);
            DEALLOCATE(IPVT);     DEALLOCATE(FACT);
        END DO
        !$omp end parallel

        logp_T(t)=sum(pp_G)


        If(t>burn) THEN
            AICM=2*(var_8(logp_T(burn:t),(t-burn+1))-sum(logp_T(burn:t))/(t-burn+1))
        END IF


        call system_clock ( clock_stop, clock_rate, clock_max )
        t_round=t/50
        IF (t-t_round*50==0) THEN
            WRITE(*,*)   'T=', t
            WRITE(*,'(a6, f7.4)') 'Time:', real ( clock_stop - clock_start ) / real ( clock_rate )
            WRITE(*,'(a6,13f7.4)') 'GAMMA:', gamma_T(:,t)
            WRITE(*,'(a6,4f7.4)') 'LAMBDA:', lambda_T(t)
            WRITE(*,'(a6,7f7.4)') 'SIGMA:',  sigma_eT(t)
            WRITE(*,'(a6,7f7.4)') 'SIGMA:',  sigma_v1T(t)
            WRITE(*,'(a6,7f7.4)') 'SIGMA:',  sigma_v2T(t)
            WRITE(*,'(a6,7f7.4)') 'SIGMA:',  sigma_v3T(t)
            WRITE(*,'(a6,2f7.4)') 'acc_0:',  acc_rate0(1,t)
            WRITE(*,'(a6,5f7.4)') 'zz_1:',  zz(1,1:DD,1)
            WRITE(*,'(a6,f7.4)') 'acc_1:',  acc_rate1(t)
            WRITE(*,'(a6,f7.4)') 'acc_2:',  acc_rate2(t)
            WRITE(*,'(a6,f15.2)') 'AICM', AICM

            IF (DD==1) THEN
                WRITE(1000,'(8f12.4)')  gamma_T(:,t)
                WRITE(3000,'(32f10.4)')  beta_yT(:,t)
                WRITE(3100,'(11f10.4)')  beta_nT(:,t)
                WRITE(3200,'(11f10.4)')  beta_cT(:,t)
                WRITE(3300,'(11f10.4)')  beta_eT(:,t)
            ELSEIF (DD==2) THEN
                WRITE(1000,'(9f12.4)')  gamma_T(:,t)
                WRITE(3000,'(34f10.4)')  beta_yT(:,t)
                WRITE(3100,'(12f10.4)')  beta_nT(:,t)
                WRITE(3200,'(12f10.4)')  beta_cT(:,t)
                WRITE(3300,'(12f10.4)')  beta_eT(:,t)
            ELSEIF (DD==3) THEN
                WRITE(1000,'(10f12.4)')  gamma_T(:,t)
                WRITE(3000,'(36f10.4)')  beta_yT(:,t)
                WRITE(3100,'(13f10.4)')  beta_nT(:,t)
                WRITE(3200,'(13f10.4)')  beta_cT(:,t)
                WRITE(3300,'(13f10.4)')  beta_eT(:,t)
            ELSEIF (DD==4) THEN
                WRITE(1000,'(11f12.4)')  gamma_T(:,t)
                WRITE(3000,'(38f10.4)')  beta_yT(:,t)
                WRITE(3100,'(14f10.4)')  beta_nT(:,t)
                WRITE(3200,'(14f10.4)')  beta_cT(:,t)
                WRITE(3300,'(14f10.4)')  beta_eT(:,t)
            ELSEIF (DD==5) THEN
                WRITE(1000,'(12f12.4)')  gamma_T(:,t)
                WRITE(3000,'(40f10.4)')  beta_yT(:,t)
                WRITE(3100,'(15f10.4)')  beta_nT(:,t)
                WRITE(3200,'(15f10.4)')  beta_cT(:,t)
                WRITE(3300,'(15f10.4)')  beta_eT(:,t)
            ELSEIF (DD==6) THEN
                WRITE(1000,'(13f12.4)')  gamma_T(:,t)
                WRITE(3000,'(42f10.4)')  beta_yT(:,t)
                WRITE(3100,'(16f10.4)')  beta_nT(:,t)
                WRITE(3200,'(16f10.4)')  beta_cT(:,t)
                WRITE(3300,'(16f10.4)')  beta_eT(:,t)
            ENDIF

            WRITE(2000,'(4f10.4)')   lambda_T(t)

            WRITE(4000,'(1f12.4)')  sigma_eT(t)
            WRITE(4100,'(1f12.4)')  sigma_v1T(t)
            WRITE(4200,'(1f12.4)')  sigma_v2T(t)
            WRITE(4300,'(1f12.4)')  sigma_v3T(t)
            !WRITE(6000,100)          alpha_yT(:,t)
            WRITE(7000,'(f12.3)')    logp_T(t)


        END IF


        !t_round=t/THIN
        !IF (t-t_round*THIN==0) THEN
        !    WRITE(5000,200)  ((zz(i,:,g),i=1,300),g=1,GG)
        !END IF


    END DO



    END PROGRAM estimation



